const HttpStatusCode = require('../enums/error-codes.enum');

class APIError extends Error {
    constructor(name, httpCode, isOperational, description) {
        super(description);
        Object.setPrototypeOf(this, new.target.prototype);

        this.name = name;
        this.httpCode = httpCode;
        this.isOperational = isOperational;

        Error.captureStackTrace(this);
    }

    static BadRequest(description) {
        return new APIError('Bad Request', HttpStatusCode.BAD_REQUEST, true, description);
    }

    static UnathorisedRequest(description) {
        return new APIError('Unathorized User!', HttpStatusCode.UNAUTHORIZED, true, description);
    }

    static NotFoundRequest(description) {
        return new APIError('Not Found', HttpStatusCode.NOT_FOUND, true, description);
    }

    static ServerError(description) {
        return new APIError('Internal Server Error', HttpStatusCode.INTERNAL_SERVER, true, description);
    }
}

module.exports = APIError;