const winston = require('winston');
require('winston-mongodb');
const EnvironmentEnum = require('./../enums/environment.enum');

const customLevels = {
  levels: {
    trace: 5,
    debug: 4,
    info: 3,
    warn: 2,
    error: 1,
    fatal: 0,
  },
  colors: {
    trace: 'white',
    debug: 'green',
    info: 'green',
    warn: 'yellow',
    error: 'red',
    fatal: 'red',
  },
};

const formatter = winston.format.combine(
  winston.format.colorize(),
  winston.format.timestamp({ format: 'YYYY-MM-DD HH:mm:ss' }),
  winston.format.splat(),
  winston.format.printf((info) => {
    const { timestamp, level, message, ...meta } = info;

    return `${timestamp} [${level}]: ${message} ${
      Object.keys(meta).length ? JSON.stringify(meta, null, 2) : ''
      }`;
  }),
);

class Logger {
  prodTransport;
  prodTransport;
  OtherTransport;
  constructor(environment, configuration) {
    if (environment.toLowerCase() === EnvironmentEnum.PROD) {
      let connectionString = configuration.connection.replace('<database>', `${configuration.errorDbName}_${environment}`);
      this.prodTransport = new winston.transports.MongoDB({
        level: 'error',
        //mongo database connection link
        db: connectionString,
        options: {
          useNewUrlParser: true,
          useUnifiedTopology: true
        },
        // A collection to save json formatted logs
        collection: 'server_logs',
        format: winston.format.combine(
          winston.format.timestamp(),
          // Convert logs to a json format
          winston.format.json())
      });
    }
    else if (environment.toLowerCase() === EnvironmentEnum.LOCAL)
      this.transport = new winston.transports.Console({
        format: formatter,
      });
    else
      this.OtherTransport = new winston.transports.File({
        filename: 'logs/error.log',
        level: 'error',
        format: winston.format.combine(
          winston.format.timestamp(),
          // Convert logs to a json format
          winston.format.json())
      });
    const logConfiguration = {
      levels: customLevels.levels,
      transports: [environment.toLowerCase() === EnvironmentEnum.LOCAL ? this.transport : environment.toLowerCase() === EnvironmentEnum.PROD ? this.prodTransport : this.OtherTransport]
    };
    // this.logger = winston.createLogger({
    //   level: isLocalEnvironment() ? 'trace' : 'error',
    //   levels: customLevels.levels,
    //   transports: [isLocalEnvironment() ? transport : prodTransport],
    // });

    this.logger = winston.createLogger(logConfiguration);
    winston.addColors(customLevels.colors);
  }

  trace(msg, meta) {
    this.logger.log('trace', msg, meta);
  }

  debug(msg, meta) {
    this.logger.debug(msg, meta);
  }

  info(msg, meta) {
    this.logger.info(msg, meta);
  }

  warn(msg, meta) {
    this.logger.warn(msg, meta);
  }

  error(msg, meta) {
    this.logger.error(msg, meta);
  }

  fatal(msg, meta) {
    this.logger.log('fatal', msg, meta);
  }
}

module.exports = Logger;